// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "CustomSessions.h"
#include "CustomSessionsDlg.h"

STDMETHODIMP CCustomSessionsDlg::OnDataError(
                           LPDISPATCH /*obj*/,
                           BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CCustomSessionsDlg::OnDataConnectionStatusChanged(
                           eConnectionStatus newStatus)
{
   try
   {
      m_lblDataConnection.SetWindowText(_T("Data ") + CHelper::ConnectionStatusToString(newStatus));

      EnableControls();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CCustomSessionsDlg::OnCustomSessionsResolved(
                           ICQGSessionsCollection* cqg_sessions_collection,
                           ICQGError* cqg_error)
{
   try
   {
      // Clearing custom sessions
      ClearCustomSessions();

      // Raise error, when any error occurred during request
      VARIANT_BOOL isValid;
      HRESULT hr = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error), &isValid);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      if (isValid)
      {
         ATL::CComBSTR errDescription;
         hr = cqg_error->get_Description(&errDescription);
         Common::ThrowOnCOMError<ICQGError>(cqg_error, hr);

         AfxMessageBox(ATL::CW2T(errDescription), MB_ICONSTOP);

         return S_OK;
      }

      // Add resolved custom sessions to the combo box
      m_spSessionsCollection = cqg_sessions_collection;

      long count;
      hr = m_spSessionsCollection->get_Count(&count);
      Common::ThrowOnCOMError(m_spSessionsCollection, hr);

      if (count > 0)
      {
         for (long i = 0; i < count; ++i)
         {
            ATL::CComPtr<ICQGSessions> spCustomSessions;
            hr = m_spSessionsCollection->get_Item(ATL::CComVariant(i), &spCustomSessions);
            Common::ThrowOnCOMError(m_spSessionsCollection, hr);

            ATL::CComBSTR name;
            hr = spCustomSessions->get_Name(&name);
            Common::ThrowOnCOMError(spCustomSessions, hr);

            int row = m_cmbCSCollection.AddString(ATL::CW2T(name));
            m_cmbCSCollection.SetItemData(row, i);
         }

         // Select first item
         m_cmbCSCollection.SetCurSel(0);
         OnCbnSelchangeCmbCsCollection();

         // Changing buttons enablements
         EnableControls();
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}
