/*
* =====================================================================
*  Filename: CustomSessionsDlg.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved.
* =====================================================================
*/

#pragma once

#include "afxwin.h"
#include "afxcmn.h"
#include "afxdtctl.h"

class CCustomSessionsDlg;

/**
\typedef ATL::IDispEventImpl<1, CCustomSessionsDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImpl
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              CCustomSessionsDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                           >
                           ICQGCELDispEventImpl;

// CCustomSessionsDlg dialog
class CCustomSessionsDlg : public CDialog, public ICQGCELDispEventImpl
{

public:

   /// \brief CCustomSessionsDlg constructor.
   /// \param pParent - parent window
   explicit CCustomSessionsDlg(
                  CWnd* pParent = NULL); 

   /// \enum Dialog Data
   enum { IDD = IDD_CUSTOM_SESSIONS_DIALOG };

private:

   /// \brief Dialog icon.
   HICON m_hIcon;

   /// \brief DDX/DDV support.
   virtual void DoDataExchange(
                  CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the framework shall set the input 
   /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

   /// \brief Override to prevent dialog from closing on Enter
   /// \brief and catch Enters in tag name and tag value edit controls.
   virtual void OnOK();

private:

   /// \brief Prepares the container for painting.
   afx_msg void OnPaint();

   /// \brief The system calls this function to obtain the cursor to
   /// \brief display while the user drags the minimized window.
   afx_msg HCURSOR OnQueryDragIcon();

   /// \brief Used to unadvise from CQGCEL events and shutdown it.
   afx_msg void OnDestroy();

   /// \brief Sets current custom sessions to selected and shows all necessary information.
   afx_msg void OnCbnSelchangeCmbCsCollection();

   /// \brief Shows all necessary information for selected custom session.
   afx_msg void OnCbnSelchangeCmbName();

   /// \brief Clears custom sessions and requests new ones.
   afx_msg void OnBnClickedBtnRequestCs();

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

public:

   /// \brief This map is used to declare handler function for specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(CCustomSessionsDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x0000000a, OnDataError)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000003, OnDataConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 0x00000014, OnCustomSessionsResolved)
   END_SINK_MAP()

private:

   /// \brief Handle event of some abnormal discrepancy
   /// \brief between data expected and data received from CQG.
   /// \brief This event also can be fired when CQGCEL startup fails.
   /// \param obj - Object where the error occurred
   /// \param errorDescription - string describing the error
   /// \return S_OK
   STDMETHOD(OnDataError)(
                  LPDISPATCH obj,
                  BSTR errorDescription);

   /// \brief Fired when some changes occurred in the connection with the CQG data server.
   /// \param newStatus - new connection status
   /// \return S_OK
   STDMETHOD(OnDataConnectionStatusChanged)(
                  eConnectionStatus newStatus);

   /// \brief Fired when the list of custom sessions is resolved or when some
   /// \brief error occurred during custom sessions request processing
   /// \param cqg_sessions_collection - collection of resolved custom sessions
   /// \param cqg_error - CQGError object describing last error occurred during processing the request
   /// \return S_OK
   STDMETHOD(OnCustomSessionsResolved)(
                  ICQGSessionsCollection* cqg_sessions_collection,
                  ICQGError* cqg_error);

private:

   /// \brief Initializes CQGCEL object, and starts the CQGCEL.
   void InitializeCQGCEL();

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Used for getting CQGCEL properties values string representation.
   /// \brief If specified value is invalid(see CQGCEL.IsValid() method) then "N/A" is returned.
   /// \param value - variant value
   /// \param valueType - variant type
   /// \return String representation of the value.
   CString GetValueAsString(
                  ATL::CComVariant value,
                  VARENUM valueType = VT_EMPTY) const;

   /// \brief Used for getting CQGCEL properties time of day string representation.
   /// \brief If specified value is invalid(see CQGCEL.IsValid() method) then "N/A" is returned.
   /// \param value - variant value
   /// \return String representation of the value.
   CString GetValueAsTimeOfDay(ATL::CComVariant value) const;

   /// \brief Enables/disables controls on the form based on connection status and other parameters.
   void EnableControls();

   /// \brief Clears all custom sessions.
   void ClearCustomSessions();

   /// \brief Clears custom session.
   void ClearCustomSession();

   /// \brief Clears data connected with session.
   void ClearCustomSessionData();

   /// \brief Initializes working days.
   void InitWorkingDays();

   /// \brief Unchecks all working days.
   void ClearWorkingDays();

   /// \brief Shows custom session working days information.
   /// \param weekDay - Custom session working days
   void ShowWorkingDays(
                  eSessionWeekDays weekDay);

private:

   /// \enum eWeekDays. Week days enum.
   enum eWeekDays;

private:

   /// \brief CQGCEL object.
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

   /// \brief Current custom sessions collection.
   ATL::CComPtr<ICQGSessionsCollection> m_spSessionsCollection;

private:

   /// \brief Data connection indicator Label control.
   CStatic m_lblDataConnection;

   /// \brief Request custom sessions Button control.
   CButton m_btnRequestCS;

   /// \brief Custom sessions' names ComboBox control.
   CComboBox m_cmbCSCollection;

   /// \brief Custom session's names ComboBox control.
   CComboBox m_cmbName;

   /// \brief Custom sessions' type Label control.
   CStatic m_lblType;

   /// \brief Custom session's number Label control.
   CStatic m_lblNumber;

   /// \brief Custom session's primary flag Label control.
   CStatic m_lblPrimaryFlag;

   /// \brief Custom session's start time Label control.
   CStatic m_lblStart;

   /// \brief Custom session's end time Label control.
   CStatic m_lblEnd;

   /// \brief Custom session's activation date Label control.
   CStatic m_lblActivationDate;

   /// \brief Custom session's working days ListView control.
   CListCtrl m_lsvWorkingDays;
};

enum CCustomSessionsDlg::eWeekDays
{
   wdSunday,
   wdMonday,
   wdTuesday,
   wdWednesday,
   wdThursday,
   wdFriday,
   wdSaturday
};
